//
//  Sample for VAD adjustment 
//

#include <assert.h>
#include <nitro.h>
#include <dwc.h>

#include "init.h"
#include "screen.h"
#include "utility.h"

#include "sound.h"

#define SAMPLING_RATE       VCT_AUDIO_FRAME_RATE    // = 8000Hz
#define MAX_SAMPLING_TIME   VCT_AUDIO_FRAME_LENGTH  // = 68ms
#define SAMPLING_BYTE       2                       // = 16bit

#define WAVE_SAMPLE ((int) (SAMPLING_RATE * MAX_SAMPLING_TIME * SAMPLING_BYTE) / 1000)

static u8 sPlayBuffer[WAVE_SAMPLE * 2] ATTRIBUTE_ALIGN(32);
static u8  sMicBuffer[WAVE_SAMPLE * 2] ATTRIBUTE_ALIGN(32);

static struct NNSSndStrm sSndStream;

/*---------------------------------------------------------------------------*
  Use VCT_GetVADStatus in sound streaming callback for check voice activity
  and level.
 *---------------------------------------------------------------------------*/
static void SndCallback(NNSSndStrmCallbackStatus sts,
                           int nChannels,
                           void* buffer[],
                           u32 length,
                           NNSSndStrmFormat format,
                           void* arg)
{
#pragma unused(nChannels, format, arg)

    static int 		curBuffer = 0;

    const void 		*micAddr;
    u32         	ofs;
    u8* 			src;

    if (sts == NNS_SND_STRM_CALLBACK_SETUP) {
        MI_CpuClear8(buffer[0], length);
        return;
    }

    //  Get mic sampling point for check pointer of the buffer 
    micAddr = MIC_GetLastSamplingAddress();
    ofs  = (u32)((u8*)micAddr - sMicBuffer);

    if ( ofs < length ) {
        src = sMicBuffer + length;
        if (curBuffer == 0) { OS_TPrintf("same buffer\n"); }
        curBuffer = 0;
    }
    else {
        src = sMicBuffer;
        if (curBuffer == 1) { OS_TPrintf("same buffer\n"); }

        curBuffer = 1;
    }

    VCT_GetVADStatus(src, length, NULL);

    //  Clear output buffer (don't play anything) 
    MI_CpuClearFast(buffer[0], length);

    //  Flush and Invalidate Cache 
    DC_FlushRange(buffer[0], length);
    DC_InvalidateRange(src, length);

    return;
}


/*---------------------------------------------------------------------------*
   Initialize mic only initial. 
 *---------------------------------------------------------------------------*/
static void InitFirst()
{
    OS_TPrintf("Init sound system\n");
    //  Initialize MIC 
    //
    MIC_Init();
    PM_Init();
    (void)PM_SetAmp(PM_AMP_ON);
    PM_SetAmpGain(PM_AMPGAIN_40);

    //  Wait for ready to MIC 
    OS_Sleep(600);

    //  Init Sound 
    //
    NNS_SndInit();
    NNS_SndStrmInit(&sSndStream);

    OS_TPrintf("Init sound system done.\n");
}

/*---------------------------------------------------------------------------*
   Initialize sound system 
 *---------------------------------------------------------------------------*/
void SetupSound()
{
    static BOOL noInit = TRUE;
  
    u8 cArray[1] = {7};
    MICAutoParam micParam;
    BOOL ret;
    NNSSndStrmFormat nns_format;
    u32 length;

    if (noInit) {
        InitFirst();
        noInit = FALSE;
    }
 
    nns_format = NNS_SND_STRM_FORMAT_PCM16;
    
    length = (u32)(SAMPLING_RATE * SAMPLING_BYTE * VCT_AUDIO_FRAME_LENGTH) / 1000;
    
    //  Initialize MIC 
    //
    micParam.type   = MIC_SAMPLING_TYPE_SIGNED_12BIT;
    micParam.buffer = sMicBuffer;
    micParam.size   = length * 2;

    micParam.rate = (u32)((NNS_SND_STRM_TIMER_CLOCK / SAMPLING_RATE) * 64);
    
    micParam.loop_enable = TRUE;
    micParam.full_callback = NULL;
    micParam.full_arg = NULL;
    MIC_StartAutoSampling(&micParam);
        
    //  Initialize sound streaming 
    //
    //
    NNS_SndStrmAllocChannel(&sSndStream, 1, cArray);
    NNS_SndStrmSetVolume(&sSndStream, 0);

    
    ret = NNS_SndStrmSetup(&sSndStream,
                           nns_format,
                           sPlayBuffer,
                           length * 2,
                           NNS_SND_STRM_TIMER_CLOCK / SAMPLING_RATE,
                           2,
                           SndCallback,
                           NULL);
    SDK_ASSERT(ret);
    NNS_SndStrmStart(&sSndStream);

    return;
}

/*---------------------------------------------------------------------------*
   Terminate streaming.
 *---------------------------------------------------------------------------*/
void TerminateSound()
{
    (void)MIC_StopAutoSampling();
    NNS_SndStrmStop(&sSndStream);
    NNS_SndStrmFreeChannel(&sSndStream);
}

